import com.j256.ormlite.dao.Dao;
import com.j256.ormlite.dao.DaoManager;
import com.j256.ormlite.jdbc.JdbcConnectionSource;
import com.j256.ormlite.support.ConnectionSource;

import java.util.List;
import java.util.Scanner;

public class AccountApp {

    // we are using a MySQl database
    private final static String DATABASE_URL = "jdbc:mysql://localhost:3306/orm_lab?useUnicode=true";

    private Dao<Account, Integer> accountDao;

    public static void main(String[] args) throws Exception {
        // turn our static method into an instance of Main
        new AccountApp().doMain(args);
    }

    private void doMain(String[] args) throws Exception {
        ConnectionSource connectionSource = null;
        try {
            // create our data-source for the database
            connectionSource = new JdbcConnectionSource(DATABASE_URL, "root", "");
            // setup our  DAOs
            setupDao(connectionSource);

            boolean flag = true;
            String command;
            while(flag) {
                command = printMenu();
                switch(command) {
                    case "1":
                        createEntry();
                        break;
                    case "2":
                        showAllEntries();
                        break;
                    case "3":
                        showById();
                        break;
                    case "4":
                        showByName();
                        break;
                    case "5":
                        showByPassword();
                        break;
                    case "6":
                        updatePasswordById();
                        break;
                    case "7":
                        updateNameAndPasswordById();
                        break;
                    case "8":
                        deleteById();
                        break;
                    case "9":
                        flag = false;
                        break;
                    default:
                        System.out.println("Wrong input! Try again");
                }
            }

        } finally {
            // destroy the data source which should close underlying connections
            if (connectionSource != null) {
                connectionSource.close();
            }
        }
    }

    private void createEntry() throws Exception {
        String name = askForInput("Enter the name (up to 20 characters):");
        String password = askForInput("Enter the password (up to 20 characters):");

        Account account = new Account(name, password);
        accountDao.create(account);

        int id = account.getId();
        System.out.println("New entry with id # " + id + " was successfully created");
    }

    private void showAllEntries() throws Exception {
        System.out.println("List of all accounts:");
        List<Account> listOfEntries = accountDao.queryForAll();
        for (Account currentAccount : listOfEntries) {
            printAccount(currentAccount);
        }
    }

    private void showByName() throws Exception {
        String name = askForInput("Enter name to lookup:");
        List<Account> account = accountDao.queryForEq("name", name);
        printList(account);
    }

    private void showById() throws Exception {
        int id = askForId("Enter ID to lookup:");
        printAccount(accountDao.queryForId(id));
    }

    private void showByPassword() throws Exception {
        String password = askForInput("Enter password to lookup:");
        List<Account> account = accountDao.queryForEq("password", password);
        printList(account);
    }

    private void deleteById() throws Exception {
        int id = askForId("Enter ID of the account you'd like to delete:");
        accountDao.deleteById(id);
        System.out.println("Entry with id # " + id + " was successfully deleted");
    }

    private void updatePasswordById() throws Exception {
        int id = askForId("Enter ID of the account you'd like to update:");
        String newPassword = askForInput("Enter a new password:");
        Account account = accountDao.queryForId(id);
        account.setPassword(newPassword);
        accountDao.update(account);
        System.out.println("Entry with id # " + id + " was successfully updated");
    }

    private void updateNameAndPasswordById() throws Exception {
        int id = askForId("Enter ID of the account you'd like to update:");
        Account account = accountDao.queryForId(id);

        String newName = askForInput("Enter a new name:");
        account.setName(newName);

        String newPassword = askForInput("Enter a new password:");
        account.setPassword(newPassword);

        accountDao.update(account);
        System.out.println("Entry with id # " + id + " was successfully updated");
    }

    private void printAccount(Account account) {
        System.out.println(String.format("%3s %20s %20s", account.getId(), account.getName(), account.getPassword()));
    }

    private void printList(List<Account> account) {
        for (Account currentAccount : account) {
            printAccount(currentAccount);
        }
    }

    private int askForId(String messageToShow) {
        Scanner scanner = new Scanner(System.in);
        System.out.println(messageToShow);
        return scanner.nextInt();
    }

    private String askForInput(String messageToShow) {
        Scanner scanner = new Scanner(System.in);
        System.out.println(messageToShow);
        return scanner.nextLine();
    }

    private String printMenu() {
        System.out.println("Please choose a command:\n" +
                "1. Create new account\n" +
                "2. Show all existing accounts\n" +
                "3. Find account by id\n" +
                "4. Find account by name\n" +
                "5. Find account by password\n" +
                "6. Update password by id\n" +
                "7. Update name and password by id\n" +
                "8. Delete account by id\n" +
                "9. Exit");
        return askForInput("");
    }

    /**
     * Setup our  DAOs
     */
    private void setupDao(ConnectionSource connectionSource) throws Exception {

        accountDao = DaoManager.createDao(connectionSource, Account.class);

    }
}